Library.LibRowdy.Setup(...);

import "core";
import "table";
import "string";
import "class";
import "ui";

--[[
	List box
]]
local ListBox = nil;
do
	local base, metatable, methods = nil, nil, { };

	-- private members

	-- public members
	methods.Add = function(self, value)
	
	end
	
	methods.Remove = function(self, value)
	
	end
	
	-- constructor
	ListBox = function(parent)
		local self = UI.FramedFrame(parent:GetName() .. "ListBox");

		self[ListBox] = {
			items = { }
		};
		
		base, metatable = class(base, metatable, methods, self, ListBox);
		
		return self;
	end
end

--[[
	Error window
]]
local ErrorWindow = nil;
do
	local base, metatable, methods = nil, nil, { };

	-- private members
	
	-- public members
	methods.AddError = function(self, errorTable)
		local protected = self[ErrorWindow];

		protected.errors[_G.Inspect.Time.Frame()] = errorTable;

		local text = "";

		if (errorTable.error) then
			text = text .. "\nError:\t" .. errorTable.error .. "\n";
		end

		if (errorTable.addon) then
			text = text .. "\nAddon:\t" .. errorTable.addon;
		end

		if (errorTable.event) then
			text = text .. "\nEvent:\t" .. errorTable.event;
		end

		if (errorTable.file) then
			text = text .. "\nFile:\t" .. errorTable.file;
		end

		if (errorTable.frame) then
			text = text .. "\nFrame:\t" .. errorTable.frame;
		end

		if (errorTable.info) then
			text = text .. "\nInfo:\t" .. errorTable.info;
		end

		if (errorTable.script) then
			text = text .. "\nScript:\t" .. errorTable.script;
		end

		if (errorTable.stacktrace) then
			local stacktrace = strtrim(errorTable.stacktrace);

			if (strstartswith(stacktrace, "stack traceback:")) then
				stacktrace = strtrim(strsub(stacktrace, 17));
			end

			text = text .. "\n\nStacktrace:\n\t" .. stacktrace;
		end

		protected.errorText:SetText(strtrim(text));

		protected.scrollFrame:UpdateContentLayout();

	end
	
	-- constructor
	ErrorWindow = function(parent)
		local self = UI.Window("Janitor.ErrorWindow", parent);
		
		self:SetTitle("Dr. Jan Itor");

		local scrollFrame = UI.ScrollFrame("Janitor.ErrorWindow.ScrollFrame", self);
		scrollFrame:SetPoint("TOPLEFT", self, "TOPLEFT", 10, 20);
		scrollFrame:SetPoint("BOTTOMRIGHT", self, "BOTTOMRIGHT", -10, -10);

		local errorText = UI.Text("Janitor.ErrorWindow.Error", self);
		errorText:SetWordwrap(true);
		errorText:SetAutoSize(true);
		errorText:SetFont("LibRowdy", "media/fonts/Inconsolata.otf");
		errorText:SetFontSize(14);
		errorText:SetFontColor(1, .4, .4, 1);
		scrollFrame:SetContent(errorText);

		self[ErrorWindow] = {
			errors = {},
			errorText = errorText,
			scrollFrame = scrollFrame
		};
		
		base, metatable = class(base, metatable, methods, self, ErrorWindow);
			
		return self;
	end
end

--[[
	Graphical error handler
]]
do
	local context = UI.Context("Janitor");
	local errorWindow = ErrorWindow(context);
	
	errorWindow:SetPoint("CENTER", _G.UIParent, "CENTER", 0, 0);
	errorWindow:SetWidth(700);
	errorWindow:SetHeight(500);
	errorWindow:SetVisible(false);
	
	function handleError(errorTable)
		--print(errorTable);
		
		errorWindow:AddError(errorTable);

		if (not errorWindow:GetVisible()) then
			errorWindow:SetVisible(true);
		end
	end

	tremovewhere(_G.Event.System.Error, function(k, v) return v[2] == "Rift"; end);
	tinsert(_G.Event.System.Error, { handleError, "Janitor", "error handler" });
end

--[[
	/print command
]]
do
	local function PrintCommand(command)
		local func, err = loadstring("return " .. command .. ";");
		
		if (func == nil) then
			error(err);
		else
			print(func());
		end
	end

	local commands = _G.Command.Slash.Register("print");

	tclear(commands);
	tinsert(commands, { PrintCommand, "Janitor", "print command" });
end