local _IDENTIFIER = (...).identifier;

--[[
	initialization
]]
local packages = { };
local addons = { };

local extractEnvironment = function()
	local environment = getfenv(3);
	
	return environment, getmetatable(environment).__index;
end

local import = function(package)
	assert(type(package) == "string", "import requires 'package' (string) parameter");

	local environment, framework = extractEnvironment();
	if (type(framework) ~= "table") then
		error("import can only import packages to rowdy managed top level functions");
	end

	local identifier = framework._IDENTIFIER;
	if (type(identifier) ~= "string") then
		error("import can only import packages to rowdy managed top level functions");
	end

	local addonMetadata = addons[identifier];
	if (type(addonMetadata) ~= "table") then
		error("addon '" .. identifier .. "' is not registered rowdy addon");
	end

	local functions = addonMetadata.packages[package];
	if (type(functions) ~= "table") then
		functions = packages[package];
	end
	if (type(functions) ~= "table") then
		error("package '" .. package .. "' not found");
	end

	for k, v in pairs(functions) do
		framework[k] = v;
	end
end

local package = function(name)
	assert(type(name) == "string", "package requires 'name' (string) parameter");

	local environment, framework = extractEnvironment();
	if (type(framework) ~= "table") then
		error("package can only create packages from rowdy managed top level functions");
	end

	for _, _ in pairs(environment) do
		error("package function must be called on empty environment");
	end

	if (packages[name] ~= nil) then
		local metatable = getmetatable(environment);
		local package = packages[name];
		local baseAccessor = setmetatable({}, { __index = metatable.__index });

		metatable.__index = function(self, key)
			local value = rawget(package, key);
			if (value ~= nil) then
				return value;
			end

			value = rawget(environment, key);
			if (value ~= nil) then
				return value;
			end

			return baseAccessor[key];
		end

		metatable.__newindex = function(self, key, value)
			package[key] = value;
		end
	else
		packages[name] = environment;
	end
end

local addonPackage = function(name)
	assert(type(name) == "string", "package requires 'name' (string) parameter");

	local environment, framework = extractEnvironment();
	if (type(framework) ~= "table") then
		error("addonPackage can only create packages from rowdy managed top level functions");
	end

	local addonMetadata = addons[identifier];
	if (type(addonMetadata) ~= "table") then
		error("addon '" .. identifier .. "' is not registered rowdy addon");
	end

	for _, _ in pairs(environment) do
		error("package function must be called on empty environment");
	end

	if (addonMetadata.packages[name] ~= nil) then
		setfenv(2, addonMetadata.packages[name]);
	else
		addonMetadata.packages[name] = environment;
	end
end

local Setup = function(info)
	local identifier = info.identifier;

	local addonMetadata = addons[identifier];
	if (type(addonMetadata) ~= "table") then
		addonMetadata = {
			addon = { },
			packages = { }
		};
		addons[identifier] = addonMetadata;

		if (identifier ~= _IDENTIFIER) then
			_G[identifier] = addonMetadata.addon;
		end
	end

	local framework = {
		_G = _G,
		_IDENTIFIER = identifier,

		import = import,
		package = package,
		addonPackage = addonPackage,

		Addon = addonMetadata.addon
	};
	local environment = setmetatable({ }, { __index = framework });
	framework._L = environment;

	setfenv(2, environment);

	return info.identifier, info.data;
end

if (not _G.Library) then _G.Library = { } end
_G.Library.LibRowdy = {
	Setup = Setup
};