Library.LibRowdy.Setup(...);

import "core";
import "table";

package "class";

--[[
	Very basic oop logic
]]
local classes = {};

-- public members
class = function(base, metatable, methods, instance, ctor)
	assert(base == nil or type(base) == "table", "Class.Initialize requires 'base' (table) parameter");
	assert(metatable == nil or type(metatable) == "table", "Class.Initialize requires 'metatable' (table) parameter");
	assert(type(methods) == "table", "Class.Initialize requires 'methods' (table) parameter");
	assert(type(instance) == "table", "Class.Initialize requires 'instance' (table) parameter");
	assert(type(ctor) == "function", "Class.Initialize requires 'ctor' (function) parameter");

	-- initialize class if required
	if (base == nil) then
		assert(classes[ctor] == nil, "Class.Initialize parameter 'ctor' (function) must be used only once");

		local baseMetatable = getmetatable(instance);

		if (baseMetatable ~= nil) then
			base = baseMetatable.__index;
			metatable = {
					__index = setmetatable(methods, baseMetatable),
					_rowdy_class = ctor,
				};
		else
			base = { };
			metatable = {
					__index = methods,
					_rowdy_class = ctor,
				};
		end

		classes[ctor] = true;
	end

	-- initialize instance
	setmetatable( instance, metatable );

	return base, metatable;
end

isInstanceOf = function(self, class)
	while (true) do
		local mt = getmetatable(self);

		if (mt == nil) then
			return false;
		end

		if (type(mt) ~= "table") then
			return false;
		end

		if (mt._rowdy_class == class) then
			return true;
		end

		self = mt.__index;
	end
end

-- test
if (false) then
	local Foo = nil;
	do
		local base, metatable, methods = nil, nil, { };
		
		-- public members
		methods.Test = function(self)
			return "Foo";
		end
		
		-- constructor
		Foo = function(self)
			local instance = { };

			base, metatable = class(base, metatable, methods, instance, Foo);

			return instance;
		end
	end
	
	local Bar = nil;
	do
		local base, metatable, methods = nil, nil, { };
		
		-- public members
		methods.Test = function(self)
			return base.Test() .. "Bar";
		end
		
		-- constructor
		Bar = function(self)
			local instance = Foo();

			base, metatable = class(base, metatable, methods, instance, Bar);

			return instance;
		end
	end
	
	local foo = Foo();
	local bar = Bar();
	
	assert(foo.Test() == "Foo", "foo.Test doesn't return proper value");
	assert(bar.Test() == "FooBar", "bar.Test doesn't return proper value");
	assert(isInstanceOf(foo, Foo) == true, "foo is not instance of Foo");
	assert(isInstanceOf(bar, Foo) == true, "bar is not instance of Foo");
	assert(isInstanceOf(bar, Bar) == true, "bar is not instance of Bar");
	assert(isInstanceOf(foo, Bar) == false, "foo is instance of Bar");
	print("Class tests passed");
end