Library.LibRowdy.Setup(...);

package "core";

tostring = _G.tostring;
type = _G.type;
select = _G.select;
unpack = _G.unpack;
assert = _G.assert;
error = _G.error;
dummy = function() end;
warn = function(message) _G.print("Warning: " .. message); end

math = _G.math;


--[[
	Iterators
]]
pairs = _G.pairs;
ipairs = _G.ipairs;
next = _G.next;
values = function(...)
	local values = { ... };

	local index = 0;
	local count = #values;

	return function()
		index = index + 1;

		if (index <= count) then
			return values[index];
		end
	end
end

--[[
	Improved print function
]]
do
	local print = _G.print;
	
	-- private members
	local function GetIndent(indent)
		local result = "";

		for i = 0, indent - 1 do
			result = result .. "    ";
		end

		return result;
	end

	local function FormatKey(key)
		local keyType = type(key);

		if (keyType == "number") then
			return "[" .. key .. "]";
		else
			return tostring(key);
		end
	end

	local function FormatValue(value)
		local valueType = type(value);

		if (valueType == "string") then
			return "\"" .. value .. "\"";
		else
			return tostring(value);
		end
	end

	-- note: this needs to be rewritten..
	local function Print(key, value, indent, recursion)
		local valueType = type(value);
		local indentChars = GetIndent(indent);

		if (valueType == "table") then
			if (recursion ~= nil and recursion[value] ~= nil) then
				if (key ~= nil) then
					print(indentChars .. FormatKey(key) .. " = <" .. tostring(value) .. ">");
				else
					print(indentChars .. "<" .. tostring(value) .. ">");
				end
			else
				recursion = recursion or {};
				recursion[value] = true;

				if (not next(value)) then
					if (key ~= nil) then
						print(indentChars .. FormatKey(key) .. " = " .. tostring(value) .. " { }");
					else
						print(indentChars .. tostring(value) .. " { }");
					end
				else
					if (key ~= nil) then
						print(indentChars .. FormatKey(key) .. " = " .. tostring(value) .. " {");
					else
						print(indentChars .. tostring(value) .. " {");
					end
					for k, v in pairs(value) do
						Print(k, v, indent + 1, recursion);
					end
					print(indentChars .. "}");
				end
			end
		else
			if (key ~= nil) then
				print(indentChars .. FormatKey(key) .. " = " .. FormatValue(value));
			else
				print(indentChars .. FormatValue(value));
			end
		end
	end

	-- public members
	_L.print = function(value, ...)
		local argCount = select("#", ...);

		if (argCount > 0) then
			Print(1, value, 0, nil);

			for i = 1, argCount do
				Print(i + 1, select(i, ...), 0, nil);
			end
		else
			Print(nil, value, 0, nil);
		end

		return value, ...;
	end
end

-- this might go to another package eventually
loadstring = _G.loadstring;
rawset = _G.rawset;
rawget = _G.rawget;
getmetatable = _G.getmetatable;
setmetatable = _G.setmetatable;
getfenv = _G.getfenv;
setfenv = _G.setfenv;
