Library.LibRowdy.Setup(...);

import "core";
import "string";
import "event";
import "class";

package "ui";

--[[
	UI frame wrapper
	Warning: this is not real class!
]]
do
	UI.FrameBase = function(frameType, name, parent)
		local self = _G.UI.CreateFrame(frameType, name, parent);

		return self;
	end
end

local SetSize = function(self, width, height)
	self:SetWidth(width);
	self:SetHeight(height);
end

local GetSize = function(self)
	return self:GetWidth(), self:GetHeight();
end

--[[
	UI frame (frame) wrapper
]]
do
	local base, metatable, methods = nil, nil, { SetSize = SetSize, GetSize = GetSize };
	local fakeBaseMetatable = nil;

	-- constructor
	UI.Frame = function(name, parent)
		assert(type(name) == "string", "UI.Frame requires 'name' (string) parameter");
		assert(isInstanceOf(parent, UI.Context) or isInstanceOf(parent, UI.FrameBase),
			"UI.Frame requires 'parent' (UI.Context or UI.FrameBase) parameter");
	
		local self = UI.FrameBase("Frame", name, parent);
		
		if (fakeBaseMetatable == nil) then
			fakeBaseMetatable = { 
				__index = setmetatable({}, getmetatable(self)),
				_rowdy_class = UI.FrameBase,
			};
		end
		
		self.Event = UIEventTable(self.Event);
		
		setmetatable(self, fakeBaseMetatable);

		base, metatable = class(base, metatable, methods, self, UI.Frame);

		return self;
	end
end

--[[
	UI frame (text) wrapper
]]
do
	local base, metatable, methods = nil, nil, { SetSize = SetSize, GetSize = GetSize };
	local fakeBaseMetatable = nil;

	-- public members
	methods.SetText = function(self, text)
		base.SetText(self, text);
		
		if (self[UI.Text].autosize) then
			self:ResizeToText();
		end
	end
	
	methods.SetAutoSize = function(self, value)
		self[UI.Text].autosize = value;
		
		if (value) then
			self:ResizeToText();
		end
	end
	
	methods.GetAutoSize = function(self, value)
		return self[UI.Text].autosize;
	end

	-- constructor
	UI.Text = function(name, parent)
		assert(type(name) == "string", "UI.Text requires 'name' (string) parameter");
		assert(isInstanceOf(parent, UI.Context) or isInstanceOf(parent, UI.FrameBase),
			"UI.Text requires 'parent' (UI.Context) parameter");
	
		local self = UI.FrameBase("Text", name, parent);
		
		if (fakeBaseMetatable == nil) then
			fakeBaseMetatable = { 
				__index = setmetatable({}, getmetatable(self)),
				_rowdy_class = UI.FrameBase,
			};
		end
		
		self.Event = UIEventTable(self.Event);
		
		setmetatable(self, fakeBaseMetatable);
		
		self[UI.Text] = {
		
		};
		
		base, metatable = class(base, metatable, methods, self, UI.Text);

		self:SetAutoSize(false);

		return self;
	end
end

--[[
	UI frame (texture) wrapper
]]
do
	local base, metatable, methods = nil, nil, { SetSize = SetSize, GetSize = GetSize };
	local fakeBaseMetatable = nil;

	-- constructor
	UI.Texture = function(name, parent)
		assert(type(name) == "string", "UI.Texture requires 'name' (string) parameter");
		assert(isInstanceOf(parent, UI.Context) or isInstanceOf(parent, UI.FrameBase),
			"UI.Texture requires 'parent' (UI.Context) parameter");
	
		local self = UI.FrameBase("Texture", name, parent);

		if (fakeBaseMetatable == nil) then
			fakeBaseMetatable = { 
				__index = setmetatable({}, getmetatable(self)),
				_rowdy_class = UI.FrameBase,
			};
		end
		
		self.Event = UIEventTable(self.Event);
		
		setmetatable(self, fakeBaseMetatable);
		
		base, metatable = class(base, metatable, methods, self, UI.Texture);

		return self;
	end
end

--[[
	UI frame (mask) wrapper
]]
do
	local base, metatable, methods = nil, nil, { SetSize = SetSize, GetSize = GetSize };
	local fakeBaseMetatable = nil;

	-- constructor
	UI.Mask = function(name, parent)
		assert(type(name) == "string", "UI.Mask requires 'name' (string) parameter");
		assert(isInstanceOf(parent, UI.Context) or isInstanceOf(parent, UI.FrameBase),
			"UI.Mask requires 'parent' (UI.Context) parameter");

		local self = UI.FrameBase("Mask", name, parent);

		if (fakeBaseMetatable == nil) then
			fakeBaseMetatable = {
				__index = setmetatable({}, getmetatable(self)),
				_rowdy_class = UI.FrameBase,
			};
		end

		self.Event = UIEventTable(self.Event);

		setmetatable(self, fakeBaseMetatable);

		base, metatable = class(base, metatable, methods, self, UI.Mask);

		return self;
	end
end

-- test
if (false) then
	local context = UI.Context("UI.Frame unit test context");
	
	local frame = UI.Frame("UI.Frame unit test", context);
	assert(isInstanceOf(frame, UI.FrameBase), "frame is not instance of UI.FrameBase");
	assert(isInstanceOf(frame, UI.Frame), "frame is not instance of UI.Frame");

	local text = UI.Text("UI.Text unit test", context);
	assert(isInstanceOf(text, UI.FrameBase), "text is not instance of UI.FrameBase");
	assert(isInstanceOf(text, UI.Text), "text is not instance of UI.Text");
	
	local texture = UI.Texture("UI.Texture unit test", context);
	assert(isInstanceOf(texture, UI.FrameBase), "texture is not instance of UI.FrameBase");
	assert(isInstanceOf(texture, UI.Texture), "texture is not instance of UI.Texture");

	local mask = UI.Mask("UI.Mask unit test", context);
	assert(isInstanceOf(mask, UI.FrameBase), "mask is not instance of UI.FrameBase");
	assert(isInstanceOf(mask, UI.Mask), "mask is not instance of UI.Mask");

	print("UI.Frame tests passed");
end