Library.LibRowdy.Setup(...);

import "core";
import "class";

package "ui";

local base, metatable, methods = nil, nil, { };

-- private members

-- public members
methods.SetMinMaxValues = function(self, min, max)
	assert(type(min) == "number", "UI.ProgressBar.SetMinMaxValues requires parameter 'min' (number)");
	assert(type(max) == "number", "UI.ProgressBar.SetMinMaxValues requires parameter 'max' (number)");
	assert(min < max, "UI.ProgressBar.SetMinMaxValues requires parameter 'min' to be smaller than parameter 'max'");

	local protected = self[UI.ProgressBar];

	protected.min = min;
	protected.max = max;

	self:Update();
end

methods.GetMinMaxValues = function(self)
	local protected = self[UI.ProgressBar];

	return protected.min, protected.max;
end

methods.SetValue = function(self, value)
	assert(type(value) == "number", "UI.ProgressBar.SetValue requires parameter 'value' (number)");

	self[UI.ProgressBar].value = value;
end

methods.GetValue = function(self)
	return self[UI.ProgressBar].value;
end

methods.SetReverse = function(self, value)
	assert(type(value) == "boolean", "UI.ProgressBar.SetReverse requires parameter 'value' (boolean)");

	self[UI.ProgressBar].reverse = value;
end

methods.GetReverse = function(self)
	return self[UI.ProgressBar].reverse;
end

methods.Update = function(self)
	local protected = self[UI.ProgressBar];

	local value = math.max(protected.min, math.min(protected.value, protected.max));
	local state = math.max(0, math.min((protected.value - protected.min) / (protected.max - protected.min), 1));

	if (protected.reverse) then
		state = 1 - state;
	end

	local width = self:GetWidth() * state;

	protected.foreground:SetWidth(width);
end

-- constructor
UI.ProgressBar = function(name, parent)
	local self = UI.Frame(name, parent);

	local background = UI.Texture(name .. "Background", self);
	background:SetAllPoints(self);
	background:SetTexture("LibRowdy", "media/textures/tukui.png");

	local backgroundMask = UI.Frame(name .. "BackgroundMask", background);
	backgroundMask:SetAllPoints(background);
	backgroundMask:SetBackgroundColor(0, 0, 0, .3);

	local foreground = UI.Texture(name .. "Foreground", backgroundMask);
	foreground:SetPoint("TOPLEFT", backgroundMask, "TOPLEFT", 0, 0);
	foreground:SetPoint("BOTTOM", backgroundMask, "BOTTOM", nil, 0);
	foreground:SetBackgroundColor(0, 0, 0, .7);

	self[UI.ProgressBar] = {
		background = background,
		foreground = foreground,
		min = 0,
		max = 100,
		value = 50,
		reverse = false
	};

	base, metatable = class(base, metatable, methods, self, UI.ProgressBar);

	self:Update();

	return self;
end