Library.LibRowdy.Setup(...);

import "core";
import "class";

package "ui";

local base, metatable, methods = nil, nil, { };

-- private members
local function OnWheelForward(self)
	self:Scroll(1);
end

local function OnWheelBack(self)
	self:Scroll(-1);
end

-- public members
methods.Scroll = function(self, delta)
	local protected = self[UI.ScrollFrame];

	local height = protected.mask:GetHeight();
	local contentHeight = protected.content:GetHeight();
	local _, offset, origin, _ = protected.content:ReadPoint("TOP");

	offset = origin + delta * protected.speed;

	if (offset > 0 or contentHeight < height) then
		offset = 0;
	elseif (contentHeight + offset < height) then
		offset = -contentHeight + height;
	end

	protected.content:ClearPoint("TOP");
	protected.content:SetPoint("TOP", protected.mask, "TOP", nil, offset);
end

methods.SetSpeed = function(self, value)
	assert(type(value) == "number", "UI.ScrollFrame.SetSpeed requires parameter 'value' (number)");

	self[UI.ScrollFrame].speed = value;
end
methods.GetSpeed = function(self)
	return self[UI.ScrollFrame].speed;
end

methods.SetContent = function(self, content)
	assert(isInstanceOf(content, UI.FrameBase), "UI.ScrollFrame.SetContent requires parameter 'content' (UI.FrameBase)");

	local protected = self[UI.ScrollFrame];
	local oldContent = protected.content;

	if (oldContent ~= nil and oldContent:GetParent() == protected.mask) then
		oldContent:SetParent(nil);
	end

	protected.content = content;

	self:UpdateContentLayout();
end

methods.UpdateContentLayout = function(self)
	local protected = self[UI.ScrollFrame];
	local mask = protected.mask;
	local content = protected.content;

	content:SetParent(mask);
	content:ClearPoint("TOP");
	content:ClearPoint("LEFT");
	content:ClearPoint("RIGHT");
	content:ClearPoint("BOTTOM");
	content:ClearWidth();
	content:SetPoint("TOP", mask, "TOP", nil, 0);
	content:SetPoint("LEFT", mask, "LEFT", 0, nil);
	content:SetWidth(mask:GetWidth());
end

-- constructor
UI.ScrollFrame = function(name, parent)
	local self = UI.Frame(name, parent);

	local mask = UI.Mask(name .. "Mask", parent);
	mask:SetPoint("TOPLEFT", self, "TOPLEFT", 0, 0);
	mask:SetPoint("BOTTOMRIGHT", self, "BOTTOMRIGHT", 0, 0);
	
	self.Event.WheelForward = OnWheelForward;
	self.Event.WheelBack = OnWheelBack;

	self[UI.ScrollFrame] = {
		content = content,
		mask = mask,
		speed = 35
	};

	base, metatable = class(base, metatable, methods, self, UI.ScrollFrame);

	return self;
end