Library.LibRowdy.Setup(...);

import "core";
import "string";
import "event";
import "class";

package "ui";

local CloseButton = nil;
do
	local base, metatable, methods = nil, nil, { };

	-- private members
	local function OnClick(self)
		self[CloseButton].target:SetVisible(false);
	end
	
	-- constructor
	CloseButton = function(parent, target)
		target = target or parent;
		
		local self = UI.Button(parent);
		self:SetAutoSize(false);
		self:SetWidth(20);
		self:SetHeight(20);
		self:SetText("x");

		self.Event.Click = OnClick;
		
		self[CloseButton] = {
			target = target, 
		};
		
		base, metatable = class(base, metatable, methods, self, CloseButton);

		return self;
	end
end

--[[
	Window
]]
do
	local base, metatable, methods = nil, nil, { };

	-- public members
	methods.SetBackgroundColor = function(self, r, g, b, a)
		local title = self[UI.Window].title;
	
		if (title ~= nil) then
			title:SetBackgroundColor(r, g, b, a);
		end
	
		base.SetBackgroundColor(self, r, g, b, a);
	end
	
	methods.SetBorderColor = function(self, r, g, b, a)
		local title = self[UI.Window].title;

		if (title ~= nil) then
			title:SetBorderColor(r, g, b, a);
		end
	
		base.SetBorderColor(self, r, g, b, a);
	end
	
	methods.SetTitle = function(self, text)
		local title = self[UI.Window].title;
		
		if (title == nil) then
			title = UI.FramedText(self:GetName().. "Title", self);
			title:SetPoint("CENTER", self, "TOPCENTER", 0, 0);
			title:SetBackgroundColor(self:GetBackgroundColor());
			title:SetBorderColor(self:GetBorderColor());
			title:SetFontSize(14);
			self[UI.Window].title = title;
		end
		
		title:SetText(text);
	end
	
	methods.SetVisible = function(self, value)
		if (base.GetVisible(self) == value) then
			return;
		end
		
		if (value) then
			self.Event.Show(self);
		else
			self.Event.Hide(self);
		end
			
		base.SetVisible(self, value);
	end

	-- constructor
	UI.Window = function(name, parent)
		assert(type(name) == "string", "UI.Window requires 'name' (string) parameter");
		assert(isInstanceOf(parent, UI.Context) or isInstanceOf(parent, UI.FrameBase),
			"UI.Window requires 'parent' (UI.Context or UI.FrameBase) parameter");
	
		local self = UI.FramedFrame(name, parent);
		
		local closeButton = CloseButton(self);
		closeButton:SetPoint("CENTERRIGHT", self, "TOPRIGHT", -10, 1);
		
		self.Event:add("Show", "Hide");
		
		self[UI.Window] = {

		};
		
		base, metatable = class(base, metatable, methods, self, UI.Window);

		return self;
	end
end